@@ -40,46 +40,47 @@ var (
 		You can use '%[1]s status' to check the progress.`)
 
 	newBuildExample = templates.Examples(`
-	  # Create a build config based on the source code in the current git repository (with a public
-	  # remote) and a Docker image
-	  %[1]s %[2]s . --docker-image=repo/langimage
+		# Create a build config based on the source code in the current git repository (with a public
+		# remote) and a Docker image
+		oc new-build . --docker-image=repo/langimage
 
-	  # Create a NodeJS build config based on the provided [image]~[source code] combination
-	  %[1]s %[2]s centos/nodejs-8-centos7~https://github.com/sclorg/nodejs-ex.git
+		# Create a NodeJS build config based on the provided [image]~[source code] combination
+		oc new-build centos/nodejs-8-centos7~https://github.com/sclorg/nodejs-ex.git
 
-	  # Create a build config from a remote repository using its beta2 branch
-	  %[1]s %[2]s https://github.com/openshift/ruby-hello-world#beta2
+		# Create a build config from a remote repository using its beta2 branch
+		oc new-build https://github.com/openshift/ruby-hello-world#beta2
 
-	  # Create a build config using a Dockerfile specified as an argument
-	  %[1]s %[2]s -D $'FROM centos:7\nRUN yum install -y httpd'
+		# Create a build config using a Dockerfile specified as an argument
+		oc new-build -D $'FROM centos:7\nRUN yum install -y httpd'
 
-	  # Create a build config from a remote repository and add custom environment variables
-	  %[1]s %[2]s https://github.com/openshift/ruby-hello-world -e RACK_ENV=development
+		# Create a build config from a remote repository and add custom environment variables
+		oc new-build https://github.com/openshift/ruby-hello-world -e RACK_ENV=development
 
-	  # Create a build config from a remote private repository and specify which existing secret to use
-	  %[1]s %[2]s https://github.com/youruser/yourgitrepo --source-secret=yoursecret
+		# Create a build config from a remote private repository and specify which existing secret to use
+		oc new-build https://github.com/youruser/yourgitrepo --source-secret=yoursecret
 
-	  # Create a build config from a remote repository and inject the npmrc into a build
-	  %[1]s %[2]s https://github.com/openshift/ruby-hello-world --build-secret npmrc:.npmrc
+		# Create a build config from a remote repository and inject the npmrc into a build
+		oc new-build https://github.com/openshift/ruby-hello-world --build-secret npmrc:.npmrc
 
-	  # Create a build config from a remote repository and inject environment data into a build
-	  %[1]s %[2]s https://github.com/openshift/ruby-hello-world --build-config-map env:config
+		# Create a build config from a remote repository and inject environment data into a build
+		oc new-build https://github.com/openshift/ruby-hello-world --build-config-map env:config
 
-	  # Create a build config that gets its input from a remote repository and another Docker image
-	  %[1]s %[2]s https://github.com/openshift/ruby-hello-world --source-image=openshift/jenkins-1-centos7 --source-image-path=/var/lib/jenkins:tmp`)
+		# Create a build config that gets its input from a remote repository and another Docker image
+		oc new-build https://github.com/openshift/ruby-hello-world --source-image=openshift/jenkins-1-centos7 --source-image-path=/var/lib/jenkins:tmp
+	`)
 
 	newBuildNoInput = `You must specify one or more images, image streams, or source code locations to create a build.
 
 To build from an existing image stream tag or Docker image, provide the name of the image and
 the source code location:
 
-  %[1]s %[2]s centos/nodejs-8-centos7~https://github.com/sclorg/nodejs-ex.git
+  oc new-build centos/nodejs-8-centos7~https://github.com/sclorg/nodejs-ex.git
 
 If you only specify the source repository location (local or remote), the command will look at
 the repo to determine the type, and then look for a matching image on your server or on the
 default container image registry.
 
-  %[1]s %[2]s https://github.com/sclorg/nodejs-ex.git
+  oc new-build https://github.com/sclorg/nodejs-ex.git
 
 will look for an image called "nodejs" in your current project, the 'openshift' project, or
 on the Docker Hub.
@@ -106,24 +107,22 @@ func NewBuildOptions(streams genericclioptions.IOStreams) *BuildOptions {
 }
 
 // NewCmdNewBuild implements the OpenShift cli new-build command
-func NewCmdNewBuild(name, baseName string, f kcmdutil.Factory, streams genericclioptions.IOStreams) *cobra.Command {
+func NewCmdNewBuild(f kcmdutil.Factory, streams genericclioptions.IOStreams) *cobra.Command {
 	o := NewBuildOptions(streams)
 
 	cmd := &cobra.Command{
-		Use:        fmt.Sprintf("%s (IMAGE | IMAGESTREAM | PATH | URL ...)", name),
+		Use:        "new-build (IMAGE | IMAGESTREAM | PATH | URL ...)",
 		Short:      "Create a new build configuration",
-		Long:       fmt.Sprintf(newBuildLong, baseName, name),
-		Example:    fmt.Sprintf(newBuildExample, baseName, name),
+		Long:       newBuildLong,
+		Example:    newBuildExample,
 		SuggestFor: []string{"build", "builds"},
 		Run: func(cmd *cobra.Command, args []string) {
-			kcmdutil.CheckErr(o.Complete(baseName, name, f, cmd, args))
+			kcmdutil.CheckErr(o.Complete(f, cmd, args))
 			kcmdutil.CheckErr(o.RunNewBuild())
 		},
 	}
 
 	cmd.Flags().StringSliceVar(&o.Config.SourceRepositories, "code", o.Config.SourceRepositories, "Source code in the build configuration.")
-	cmd.Flags().StringSliceVarP(&o.Config.ImageStreams, "image", "", o.Config.ImageStreams, "Name of an image stream to to use as a builder. (deprecated)")
-	cmd.Flags().MarkDeprecated("image", "use --image-stream instead")
 	cmd.Flags().StringSliceVarP(&o.Config.ImageStreams, "image-stream", "i", o.Config.ImageStreams, "Name of an image stream to to use as a builder.")
 	cmd.Flags().StringSliceVar(&o.Config.DockerImages, "docker-image", o.Config.DockerImages, "Name of a Docker image to use as a builder.")
 	cmd.Flags().StringSliceVar(&o.Config.ConfigMaps, "build-config-map", o.Config.ConfigMaps, "ConfigMap and destination to use as an input for the build.")
@@ -141,11 +140,12 @@ func NewCmdNewBuild(name, baseName string, f kcmdutil.Factory, streams genericcl
 	cmd.Flags().MarkHidden("build-env-file")
 	cmd.Flags().StringArrayVar(&o.Config.BuildEnvironmentFiles, "env-file", o.Config.BuildEnvironmentFiles, "File containing key-value pairs of environment variables to set into each container.")
 	cmd.MarkFlagFilename("env-file")
-	cmd.Flags().Var(&o.Config.Strategy, "strategy", "Specify the build strategy to use if you don't want to detect (docker|pipeline|source).")
+	cmd.Flags().Var(&o.Config.Strategy, "strategy", "Specify the build strategy to use if you don't want to detect (docker|pipeline|source). NOTICE: the pipeline strategy is deprecated; consider using Jenkinsfiles directly on Jenkins or OpenShift Pipelines.")
 	cmd.Flags().StringVarP(&o.Config.Dockerfile, "dockerfile", "D", o.Config.Dockerfile, "Specify the contents of a Dockerfile to build directly, implies --strategy=docker. Pass '-' to read from STDIN.")
 	cmd.Flags().StringArrayVar(&o.Config.BuildArgs, "build-arg", o.Config.BuildArgs, "Specify a key-value pair to pass to Docker during the build.")
 	cmd.Flags().BoolVar(&o.Config.BinaryBuild, "binary", o.Config.BinaryBuild, "Instead of expecting a source URL, set the build to expect binary contents. Will disable triggers.")
 	cmd.Flags().StringP("labels", "l", "", "Label to set in all generated resources.")
+	cmd.Flags().BoolVar(&o.Config.InsecureRegistry, "insecure-registry", o.Config.InsecureRegistry, "If true, indicates that the referenced Docker images are on insecure registries and should bypass certificate checking")
 	cmd.Flags().BoolVar(&o.Config.AllowMissingImages, "allow-missing-images", o.Config.AllowMissingImages, "If true, indicates that referenced Docker images that cannot be found locally or in a registry should still be used.")
 	cmd.Flags().BoolVar(&o.Config.AllowMissingImageStreamTags, "allow-missing-imagestream-tags", o.Config.AllowMissingImageStreamTags, "If true, indicates that image stream tags that don't exist should still be used.")
 	cmd.Flags().StringVar(&o.Config.ContextDir, "context-dir", o.Config.ContextDir, "Context directory to be used for the build.")
@@ -161,8 +161,8 @@ func NewCmdNewBuild(name, baseName string, f kcmdutil.Factory, streams genericcl
 }
 
 // Complete sets any default behavior for the command
-func (o *BuildOptions) Complete(baseName, commandName string, f kcmdutil.Factory, cmd *cobra.Command, args []string) error {
-	err := o.ObjectGeneratorOptions.Complete(baseName, commandName, f, cmd, args)
+func (o *BuildOptions) Complete(f kcmdutil.Factory, cmd *cobra.Command, args []string) error {
+	err := o.ObjectGeneratorOptions.Complete(f, cmd, args)
 	if err != nil {
 		return err
 	}
@@ -187,7 +187,7 @@ func (o *BuildOptions) RunNewBuild() error {
 
 	result, err := config.Run()
 	if err != nil {
-		return ocnewapp.HandleError(err, o.BaseName, o.CommandName, o.CommandPath, config, transformBuildError)
+		return ocnewapp.HandleError(err, o.CommandPath, config, transformBuildError)
 	}
 
 	if len(config.Labels) == 0 && len(result.Name) > 0 {
@@ -227,9 +227,12 @@ func (o *BuildOptions) RunNewBuild() error {
 	for _, item := range result.List.Items {
 		switch t := item.(type) {
 		case *buildv1.BuildConfig:
+			if t.Spec.Strategy.Type == buildv1.JenkinsPipelineBuildStrategyType {
+				fmt.Fprintln(o.Action.ErrOut, "JenkinsPipeline build strategy is deprecated. Use Jenkinsfiles directly on Jenkins or OpenShift Pipelines instead")
+			}
 			if len(t.Spec.Triggers) > 0 && t.Spec.Source.Binary == nil {
 				fmt.Fprintf(out, "%sBuild configuration %q created and build triggered.\n", indent, t.Name)
-				fmt.Fprintf(out, "%sRun '%s logs -f bc/%s' to stream the build progress.\n", indent, o.BaseName, t.Name)
+				fmt.Fprintf(out, "%sRun 'oc logs -f buildconfig/%s' to stream the build progress.\n", indent, t.Name)
 			}
 		}
 	}
@@ -237,7 +240,7 @@ func (o *BuildOptions) RunNewBuild() error {
 	return nil
 }
 
-func transformBuildError(err error, baseName, commandName, commandPath string, groups ocnewapp.ErrorGroups, config *newcmd.AppConfig) {
+func transformBuildError(err error, commandPath string, groups ocnewapp.ErrorGroups, config *newcmd.AppConfig) {
 	switch t := err.(type) {
 	case newapp.ErrNoMatch:
 		classification, _ := config.ClassificationWinners[t.Value]
@@ -269,9 +272,9 @@ func transformBuildError(err error, baseName, commandName, commandPath string, g
 	}
 	switch err {
 	case newcmd.ErrNoInputs:
-		groups.Add("", "", "", ocnewapp.UsageError(commandPath, newBuildNoInput, baseName, commandName))
+		groups.Add("", "", "", ocnewapp.UsageError(commandPath, newBuildNoInput))
 		return
 	}
-	ocnewapp.TransformRunError(err, baseName, commandName, commandPath, groups, config)
+	ocnewapp.TransformRunError(err, commandPath, groups, config)
 	return
 }
